"use strict";
var binding_count_1 = require("../bindings/binding_count");
var lookup_1 = require("./lookup");
var planner_1 = require("../planning/planner");
var resolver_1 = require("../resolution/resolver");
var ERROR_MSGS = require("../constants/error_msgs");
var Kernel = (function () {
    function Kernel() {
        var middleWare = [];
        this._parentKernel = null;
        this._planner = new planner_1.Planner();
        this._resolver = new resolver_1.Resolver(middleWare);
        this._bindingDictionary = new lookup_1.Lookup();
    }
    Kernel.prototype.bind = function (typeBinding) {
        this._bindingDictionary.add(typeBinding.runtimeIdentifier, typeBinding);
    };
    Kernel.prototype.unbind = function (runtimeIdentifier) {
        try {
            this._bindingDictionary.remove(runtimeIdentifier);
        }
        catch (e) {
            throw new Error(ERROR_MSGS.CANNOT_UNBIND + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.unbindAll = function () {
        this._bindingDictionary = new lookup_1.Lookup();
    };
    Kernel.prototype.get = function (runtimeIdentifier) {
        var bindings = this._getBindingsByRuntimeIdentifier(runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.BindingCountEnum.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.BindingCountEnum.OnlyOneBindingAvailable:
                return this._planAndResolve(bindings[0]);
            case binding_count_1.BindingCountEnum.MultipleBindingsAvailable:
            default:
                throw new Error(ERROR_MSGS.AMBIGUOUS_MATCH + " " + runtimeIdentifier);
        }
    };
    Kernel.prototype.getAll = function (runtimeIdentifier) {
        var _this = this;
        var bindings = this._getBindingsByRuntimeIdentifier(runtimeIdentifier);
        switch (bindings.length) {
            case binding_count_1.BindingCountEnum.NoBindingsAvailable:
                throw new Error(ERROR_MSGS.NOT_REGISTERED + " " + runtimeIdentifier);
            case binding_count_1.BindingCountEnum.OnlyOneBindingAvailable:
            case binding_count_1.BindingCountEnum.MultipleBindingsAvailable:
            default:
                return bindings.map(function (binding) {
                    return _this._planAndResolve(binding);
                });
        }
    };
    Kernel.prototype._getBindingsByRuntimeIdentifier = function (runtimeIdentifier) {
        var bindings = [];
        if (this._bindingDictionary.hasKey(runtimeIdentifier)) {
            bindings = this._bindingDictionary.get(runtimeIdentifier);
        }
        return bindings;
    };
    Kernel.prototype._planAndResolve = function (binding) {
        var context = this._planner.createContext(this);
        var plan = this._planner.createPlan(context, binding);
        context.addPlan(plan);
        return this._resolver.resolve(context);
    };
    return Kernel;
}());
exports.Kernel = Kernel;
